////////////////////////////////////////////////////////////////////////////////
//
//	calib.c
//
//	Alain BARTHE
//
//	Shared library to access solarOrbiter calibrations from Python
//
////////////////////////////////////////////////////////////////////////////////

#define	PY_SSIZE_T_CLEAN

#define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION

#include <Python.h>
#include <stdlib.h>

#include "numpy/arrayobject.h"
#include "datetime.h"

#include "TOOLS.h"
#include "CALIB.h"

#define	MAX_DIMS	6

typedef struct {				// calibration table structure
	char *		key;			// key described in catalog
	PyObject *	object;			// numpy array object
	void *		data;			// C global variable array
	int		type;			// numpy dtype (NPY_FLoAT)
	int		ndim;			// number of array dimension
	npy_intp	dims [MAX_DIMS];	// sizes of each dimension
}	t_calib_table;

static	t_calib_table	tables [] = {
	{ "AZS",	NULL,	& AZS,	NPY_FLOAT, 2, { 11, 5		}},
	{ "AZF",	NULL,	& AZF, 	NPY_FLOAT, 3, { 11, 9, 5 	}},
	{ "ELS",	NULL, 	& ELS,	NPY_FLOAT, 2, {  9, 5 		}},
	{ "ELF",	NULL, 	& ELF,	NPY_FLOAT, 3, { 11, 9, 5	}},
	{ "ELD",	NULL,	& ELD,	NPY_FLOAT, 1, { 96 		}},
	{ "EVS",	NULL,	& EVS,	NPY_FLOAT, 2, { 96, 3		}},
	{ "EVF",	NULL,	& EVF,	NPY_FLOAT, 4, { 11, 9, 96, 3	}},
	{ "GFF",	NULL,	& GFF,	NPY_FLOAT, 2, { 11, 9		}},
	{ "CNF",	NULL, 	& CNF,	NPY_FLOAT, 3, { 11, 9, 96	}},
	{ "YIL",	NULL,	& YIL,	NPY_FLOAT, 4, { 11, 9, 11, 9	}},
	{ NULL,		NULL,	NULL,	NPY_FLOAT, 0, { 0		}}
};


////////////////////////////////////////////////////////////////////////////////
//
//	Search calibration key entry
//
static 	
t_calib_table * search_key (char * key)
{
	t_calib_table *	ptr;
	
	for (ptr = tables; ptr->key != NULL; ptr++)
		if (strcmp (key, ptr->key) == 0) return ptr;
	return NULL;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Create numpy multi-dimension array for a calibration table
//
static	
PyObject * make_numpy_array (t_calib_table * ptr)
{
	return PyArray_SimpleNewFromData (ptr->ndim, ptr->dims, ptr->type, ptr->data);
}


////////////////////////////////////////////////////////////////////////////////
//
//	Init calibration catalog
//
static 
PyObject * calib_init (PyObject * self, PyObject * args)
{	
	t_err		error = OK;
	t_calib_table *	ptr;

	if (! PyArg_ParseTuple (args, ""))
		return NULL;

	for (ptr = tables; ptr->key != NULL; ptr++) {
		ptr->object = NULL;
	}

	if (Error (error = Init_calib ("PFM"))) goto EXIT;
	
	Py_INCREF (Py_None);

	return Py_None;

EXIT:	return NULL;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Convert python datetime to Unix timestamp
//
static
time_t	PyDateTime_to_time (PyObject * obj)
{
	return (time_t) PyFloat_AsDouble (PyObject_CallMethod (obj, "timestamp", ""));
}


////////////////////////////////////////////////////////////////////////////////
//
//	Read calibration data for a given timetag and calibration key
//
static 
PyObject * calib_get (PyObject * self, PyObject * args)
{
	char *		key;
	PyObject *	obj;
	time_t		time;
	t_err		error = OK;
	t_calib_table *	ptr;

	if (! PyArg_ParseTuple (args, "Os", & obj, & key)) 
		return NULL;

	if (! PyDateTime_Check (obj)) {
		PyErr_SetString (PyExc_TypeError, "Argument must be a datetime.datetime");
		goto EXIT;
	}

	time = PyDateTime_to_time (obj);

	//	Read calibration table content and feed C global variable
	//
	if (Error (error = Read_calib (key, time))) {

		PyErr_SetString (PyExc_ValueError, "No calibration for key");
		goto EXIT;
	}

	if ((ptr = search_key (key)) == NULL) {
		PyErr_SetString (PyExc_ValueError, "Key not found");
		goto EXIT;
	}

	//	Create numpy array that point to C global data
	//	
	if (ptr->object == NULL) 
		ptr->object = make_numpy_array (ptr);

	Py_INCREF (ptr->object);

	return ptr->object;
	
EXIT:	return NULL;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Return calibration catalog filename
//
static 
PyObject * calib_get_catalog (PyObject * self, PyObject * args)
{
	if (! PyArg_ParseTuple (args, ""))
		return NULL;

	return Py_BuildValue ("s", Get_catalog_filename());
}


////////////////////////////////////////////////////////////////////////////////
//
//	Test purpose function
//
static
PyObject * calib_test (PyObject * self, PyObject * args)
{
	PyObject *	obj;
	double		d;
	time_t		t;

	if (! PyArg_ParseTuple (args, "O", & obj)) {
		return NULL;
	}

	if (! PyDateTime_Check (obj)) {
		PyErr_SetString (PyExc_TypeError, "Argument must be datetime.datetime");
		return NULL;
	}

	d = PyFloat_AsDouble (PyObject_CallMethod (obj, "timestamp", ""));
	
	t = (time_t) d;

	if (Error (Read_calib ("AZS", t))) return NULL;
	
	return Py_None;
}
	


////////////////////////////////////////////////////////////////////////////////
//
//	List of library callable methods
//	
//
static	PyMethodDef fonctions [] = {

	{ "init",		calib_init,		METH_VARARGS,	"Initialise calibration catalog\n"  },
	{ "read",		calib_get,		METH_VARARGS,	"read(datetime, key) -- return numpy array\n"},
	{ "catalog",		calib_get_catalog,	METH_VARARGS,	"catalog() -- return calibration catalog filename\n"},
	{ "test",		calib_test,		METH_VARARGS,	"test() -- test purpose function\n"},
	{ NULL, NULL, 0, NULL }
};

const char module_docstring [] = "Solar Orbiter Calibration library\n";

////////////////////////////////////////////////////////////////////////////////
//
//	Initial loading of the library
//
// 	Separate behaviour for Python 3.x and Python 2.x
//
#if PY_MAJOR_VERSION >= 3

static struct PyModuleDef moduledef = {
	PyModuleDef_HEAD_INIT,
	"PAS_calib",
	module_docstring,
	-1,
	fonctions,
};
		
#define INITERROR return NULL

PyObject *PyInit_PAS_calib(void)
{
	PyObject *module = PyModule_Create (& moduledef);

	PyDateTime_IMPORT;

	import_array ();

	PyObject_CallMethod (module, "init", "");
		
	return module;
}

#else

#define	INITERROR return

PyMODINIT_FUNC initPAS_calib (void)
{
	(void) Py_InitModule ("PAS_calib", fonctions);

	import_array ();
}
#endif
