////////////////////////////////////////////////////////////////////////////////
//
//	Module	CATALOG
//
//	SWA PAS calibration parameters catalog
//
////////////////////////////////////////////////////////////////////////////////

#define	MODULE_NAME	"CATALOG"

#include "TOOLS.h"
#include "CALIB.h"
#include "CATALOG.h"

#define	BUFSIZE		512

static	char *		calib_dir = NULL;	// Calibration directory

static	t_filename	calib_catalog;		// Calibration catalog

typedef	struct {
	char *		key;			// Calibration key
	t_filename	filename;		// Current filename
	time_t		start;			// Start of validity
	time_t		stop;			// End of validity
	P_FUNC		reader;			// Calibration reader
	int		updated;		// Updated flag

}	t_entry;

static	t_entry 	catalog [] = {

	"AZS",	"",	0, 0,	NULL, 	FALSE,
	"AZF",	"",	0, 0,	NULL, 	FALSE,
	"CNF",	"",	0, 0,	NULL, 	FALSE,
	"ELS", 	"",	0, 0,	NULL, 	FALSE,
	"ELF",	"",	0, 0,	NULL, 	FALSE,
	"ELD",	"",	0, 0,	NULL, 	FALSE,
	"EVS",	"",	0, 0,	NULL, 	FALSE,
	"EVF",	"",	0, 0,	NULL, 	FALSE,
	"GFF",	"",	0, 0,	NULL, 	FALSE,
	"YIL",	"",	0, 0,	NULL, 	FALSE,
};

static	int		nb_entry = sizeof (catalog) / sizeof (t_entry);

#define	MAX_VALIDITY	"2099-01-01T00:00:00Z"


////////////////////////////////////////////////////////////////////////////////
//
//	Search calibration key entry
//
static
t_entry * Get_entry (char * key)
{
	int		i;
	t_entry *	entry = catalog;

	for (i = 0; i < nb_entry; i++, entry++) {

		if (strcmp (key, entry->key) == 0) return entry;
	}
	return NULL;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Set calibration file reader for a given key
//
t_err	Set_reader (char * key, P_FUNC reader)
{
	char *		fname = FNAME ("Set_reader");
	t_err		error = OK;
	t_entry *	entry = NULL;

	if ((entry = Get_entry (key)) == NULL) {

		Error_log (fname, "Unkown key : %s", key);
		error = ERROR;
		goto EXIT;
	}

	entry->reader = reader;

EXIT:	return error;
}

	
////////////////////////////////////////////////////////////////////////////////
//
//	Return calibration filename for a given key
//
char *	Get_calib_filename (char * key)
{
	t_entry	* entry = Get_entry (key);

	return entry ? entry->filename : "";
}


////////////////////////////////////////////////////////////////////////////////
//
//	Return end of calibration validity for a given key
//
time_t 	Get_calib_validity (char * key)
{
	t_entry * entry = Get_entry (key);

	return entry ? entry->stop : 0;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Check if calibration is still valid
//
int	Check_calib_validity (char * key, time_t time)
{
	t_entry *	entry = Get_entry (key);

	if (entry == NULL) return FALSE;

	if (time > entry->stop) return FALSE;
	if (time < entry->start) return FALSE;

	entry->updated = FALSE;

	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Read calibration catalog
//
t_err	Read_catalog (char * key, time_t time)
{
	char *		fname = FNAME ("Read_catalog");
	t_err		error = OK;
	FILE *		catalog = NULL;
	char		buffer [BUFSIZE];
	t_entry *	entry = NULL;
	time_t		max_time = str_to_time (MAX_VALIDITY);

	if ((entry = Get_entry (key)) == NULL) {

		Error_log (fname, "No catalog entry for key %s", key);	
		error = ERR_open;
		goto EXIT;
	}

	if ((catalog = fopen (calib_catalog, "r")) == NULL) {

		Error_log (fname, "Error opening %s", calib_catalog);
		error = ERR_read;
		goto EXIT;
	}
	
	Message_log (1, fname, "Search %s calibration for %s", key, time_to_str (time));

	entry->start = 0;

	while (Read_line (catalog, BUFSIZE, buffer) != WAR_eof) {

		char		str_date [40], skey [10];
		t_filename	filename;
		time_t		start;
		int		count;

		if (strlen (buffer) == 0) continue;
		if (buffer [0] == '#') continue;

		count = sscanf (buffer, "%s %s %s", str_date, skey, filename);

		if (count != 3) continue;

		if (strcmp (skey, key) != 0) continue;

		start = str_to_time (str_date);

		if (start <= time) {
			entry->start = start;
			entry->stop  = max_time;
			sprintf (entry->filename, "%s/%s",calib_dir, filename);
		}

		if (start > time) {
			entry->stop = start;
			break;
		}
	}

	if (entry->start == 0) {

		Error_log (fname, "No calibration file for %s at %s", key, time_to_str (time));
		error = ERROR;
		goto EXIT;
	}

	Message_log (3, fname, "Read %s valid until %s", entry->filename, time_to_str (entry->stop));

	error = (* entry->reader) (entry->filename);

EXIT:	if (catalog != NULL) fclose (catalog);
	return error;
}



////////////////////////////////////////////////////////////////////////////////
//
//	Return catalog name
//
char *	Get_catalog_filename ()
{
	return calib_catalog;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Set calibration catalog
//	
t_err	Init_catalog()
{
	char *		fname = FNAME ("Init_catalog");
	t_err		error= OK;
	t_filename	pattern;
	int		count;

	if ((calib_dir = getenv ("CALIB_DIR")) == NULL) {

		Error_log (fname, "Variable CALIB_DIR not set");
		error = ERROR;
		goto EXIT;
	}

	sprintf (pattern, "%s/solo_CAL_catalog_????????_V??.txt", calib_dir);

	if (Error (error = Search_files (pattern, & count))) goto EXIT;
	
	if (count == 0) {
		Error_log (fname, "No files for %s", pattern);
		error = ERROR;
		goto EXIT;
	}

	strcpy (calib_catalog, Get_filename (count -1));

	Message_log (1, fname, "Catalog = %s", calib_catalog);

EXIT:	return error;
}
